@@ -1,5 +1,7 @@
 r"""Plotting helpers."""
 
+__all__ = ['nice_rc', 'corner', 'mark_point', 'coverage_plot']
+
 import matplotlib as mpl
 import matplotlib.pyplot as plt
 import numpy as np
@@ -8,9 +10,6 @@ from numpy import ndarray as Array
 from typing import *
 
 
-__all__ = ['nice_rc', 'corner', 'rank_ecdf']
-
-
 def nice_rc(latex: bool = False) -> Dict[str, Any]:
     r"""Returns a dictionary of runtime configuration (rc) settings for nicer
     :mod:`matplotlib` plots. The settings include 12pt font size, higher DPI,
@@ -25,23 +24,21 @@ def nice_rc(latex: bool = False) -> Dict[str, Any]:
 
     rc = {
         'axes.axisbelow': True,
-        'axes.linewidth': .8,
-        'figure.autolayout': True,
+        'axes.linewidth': 0.8,
         'figure.dpi': 150,
         'figure.figsize': (6.4, 4.8),
-        'font.size': 12.,
+        'font.size': 12.0,
         'legend.fontsize': 'x-small',
-        'lines.linewidth': 1.,
-        'lines.markersize': 3.,
-        'savefig.bbox': 'tight',
+        'lines.linewidth': 1.0,
+        'lines.markersize': 3.0,
         'savefig.transparent': True,
         'xtick.labelsize': 'x-small',
-        'xtick.major.width': .8,
+        'xtick.major.width': 0.8,
         'ytick.labelsize': 'x-small',
-        'ytick.major.width': .8,
+        'ytick.major.width': 0.8,
     }
 
-    if mpl.checkdep_usetex(latex):
+    if latex:
         rc.update({
             'font.family': ['serif'],
             'font.serif': ['Computer Modern'],
@@ -65,7 +62,7 @@ class LinearAlphaColormap(mpl.colors.LinearSegmentedColormap):
         self,
         color: Union[str, tuple],
         levels: Array = None,
-        alpha: Tuple[float, float] = (0., 1.),
+        alpha: Tuple[float, float] = (0.0, 1.0),
         name: str = None,
     ):
         if name is None:
@@ -75,18 +72,14 @@ class LinearAlphaColormap(mpl.colors.LinearSegmentedColormap):
                 name = f'alpha_{hash(color)}'
 
         if levels is None:
-            levels = [0., 1.]
+            levels = [0.0, 1.0]
 
         levels = np.asarray(levels)
         levels = (levels - levels.min()) / (levels.max() - levels.min())
         alpha = np.linspace(*alpha, len(levels))
 
         rgb = mpl.colors.to_rgb(color)
-
-        colors = sorted([
-            (l, rgb + (a,))
-            for l, a in zip(levels, alpha)
-        ])
+        colors = sorted((l, rgb + (a,)) for l, a in zip(levels, alpha))
 
         return mpl.colors.LinearSegmentedColormap.from_list(
             name=name,
@@ -94,7 +87,7 @@ class LinearAlphaColormap(mpl.colors.LinearSegmentedColormap):
         )
 
 
-def gaussian_blur(img: Array, sigma: float = 1.) -> Array:
+def gaussian_blur(img: Array, sigma: float = 1.0) -> Array:
     r"""Applies a Gaussian blur to an image.
 
     Arguments:
@@ -106,14 +99,14 @@ def gaussian_blur(img: Array, sigma: float = 1.) -> Array:
 
     Example:
         >>> img = np.random.rand(128, 128)
-        >>> gaussian_blur(img, sigma=2.)
+        >>> gaussian_blur(img, sigma=2.0)
         array([...])
     """
 
     size = 2 * int(3 * sigma) + 1
 
     k = np.arange(size) - size / 2
-    k = np.exp(-k ** 2 / (2 * sigma ** 2))
+    k = np.exp(-(k**2) / (2 * sigma**2))
     k = k / np.sum(k)
 
     smooth = lambda x: np.convolve(x, k, mode='same')
@@ -141,14 +134,13 @@ def credible_levels(hist: Array, creds: Array) -> Array:
 
 def corner(
     data: Array,
-    bins: Union[int, List[int]] = 100,
-    bounds: Tuple[Array, Array] = None,
-    creds: Array = [.6827, .9545, .9973],
+    domain: Tuple[Array, Array] = None,
+    bins: Union[int, List[int]] = 64,
+    creds: Array = [0.6827, 0.9545, 0.9973],
     color: Union[str, tuple] = None,
-    alpha: Tuple[float, float] = (0., .5),
+    alpha: Tuple[float, float] = (0.0, 0.5),
     legend: str = None,
     labels: List[str] = None,
-    markers: List[Array] = [],
     smooth: float = 0,
     figure: mpl.figure.Figure = None,
     **kwargs,
@@ -159,14 +151,13 @@ def corner(
 
     Arguments:
         data: Multi-dimensional data, either as a table or as a matrix of histograms.
+        domain: A pair of lower and upper domain bounds. If :py:`None`, inferred from data.
         bins: The number(s) of bins per dimension.
-        bounds: A tuple of lower and upper domain bounds. If :py:`None`, inferred from data.
         creds: The region credibilities (in :math:`[0, 1]`) to delimit.
         color: A color for histograms.
         alpha: A transparency range.
         legend: A legend.
         labels: The dimension labels.
-        markers: A list of points to mark on the histograms.
         smooth: The standard deviation of the smoothing kernels.
         figure: A corner plot over which to draw the new one.
         kwargs: Keyword arguments passed to :func:`matplotlib.pyplot.subplots`.
@@ -177,15 +168,16 @@ def corner(
     Example:
         >>> data = np.random.randn(2**16, 3)
         >>> labels = [r'$\alpha$', r'$\beta$', r'$\gamma$']
-        >>> corner(data, bins=42, labels=labels, figsize=(4.8, 4.8))
+        >>> figure = corner(data, bins=32, labels=labels, figsize=(4.8, 4.8))
 
-    .. image:: ../static/images/corner.png
+    .. image:: ../images/corner.png
         :align: center
         :width: 600
     """
 
     # Histograms
-    data = np.asarray(data)
+    if not isinstance(data, Array):
+        data = np.asarray(data, dtype=object)
 
     if np.isscalar(data[0, 0]):
         D = data.shape[-1]
@@ -194,10 +186,10 @@ def corner(
         if type(bins) is int:
             bins = [bins] * D
 
-        if bounds is None:
+        if domain is None:
             lower, upper = data.min(axis=0), data.max(axis=0)
         else:
-            lower, upper = map(np.asarray, bounds)
+            lower, upper = map(np.asarray, domain)
 
         bins = [
             np.histogram_bin_edges(data, bins[i], range=(lower[i], upper[i]))
@@ -216,7 +208,8 @@ def corner(
                     )
                 else:
                     hist, _, _ = np.histogram2d(
-                        data[..., i], data[..., j],
+                        data[..., i],
+                        data[..., j],
                         bins=(bins[i], bins[j]),
                         density=True,
                     )
@@ -225,10 +218,10 @@ def corner(
     else:
         D = len(data)
 
-        if bounds is None:
+        if domain is None:
             lower, upper = np.zeros(D), np.ones(D)
         else:
-            lower, upper = map(np.asarray, bounds)
+            lower, upper = map(np.asarray, domain)
 
         bins = [None] * D
         for i in range(D):
@@ -242,10 +235,11 @@ def corner(
         kwargs.setdefault('figsize', (6.4, 6.4))
 
         figure, axes = plt.subplots(
-            D, D,
+            D,
+            D,
             squeeze=False,
             sharex='col',
-            gridspec_kw={'wspace': 0., 'hspace': 0.},
+            gridspec_kw={'wspace': 0.0, 'hspace': 0.0},
             **kwargs,
         )
         new = True
@@ -255,30 +249,31 @@ def corner(
 
     # Legend
 
-    ## Color
-    lines = axes[0, -1].plot([], [], color=color, label=legend)
-    color = lines[-1].get_color()
-
-    handles, texts = axes[0, -1].get_legend_handles_labels()
-
     ## Quantiles
     creds = np.sort(np.asarray(creds))[::-1]
     creds = np.append(creds, 0)
 
-    cmap = LinearAlphaColormap('black', levels=creds, alpha=alpha)
+    if new:
+        cmap = LinearAlphaColormap('black', levels=creds, alpha=alpha)
 
-    levels = (creds - creds.min()) / (creds.max() - creds.min())
-    levels = (levels[:-1] + levels[1:]) / 2
+        levels = (creds - creds.min()) / (creds.max() - creds.min())
+        levels = (levels[:-1] + levels[1:]) / 2
 
-    for c, l in zip(creds[:-1], levels):
-        handles.append(mpl.patches.Patch(color=cmap(l), linewidth=0))
-        texts.append(r'${:.1f}\,\%$'.format(c * 100))
+        for c, l in zip(creds[:-1], levels):
+            axes[0, -1].plot(
+                [],
+                [],
+                color=cmap(l),
+                linewidth=6,
+                solid_capstyle='butt',
+                label=r'${:.1f}\,\%$'.format(c * 100),
+            )
 
-    ## Update
-    if not new:
-        figure.legends.clear()
+    ## Color
+    lines = axes[0, -1].plot([], [], color=color, solid_capstyle='butt', label=legend)
+    color = lines[-1].get_color()
 
-    figure.legend(handles, texts, loc='upper right', bbox_to_anchor=(0.975, 0.975), frameon=False)
+    axes[0, -1].legend(loc='upper right', frameon=False)
 
     # Plot
     for i in range(D):
@@ -305,16 +300,17 @@ def corner(
                 ax.plot(x, hist, color=color)
 
                 _, top = ax.get_ylim()
-                bottom = 0.
                 top = max(top, hist.max() * 1.0625)
 
                 ax.set_xlim(left=bins[i][0], right=bins[i][-1])
-                ax.set_ylim(bottom=bottom, top=top)
+                ax.set_ylim(bottom=0.0, top=top)
             else:
                 levels = np.unique(credible_levels(hist, creds))
 
                 cf = ax.contourf(
-                    x, y, hist,
+                    x,
+                    y,
+                    hist,
                     levels=levels,
                     cmap=LinearAlphaColormap(color, levels, alpha=alpha),
                 )
@@ -325,36 +321,12 @@ def corner(
                 else:
                     ax.set_ylim(bottom=bins[i][0], top=bins[i][-1])
 
-            ## Markers
-            for marker in map(np.asarray, markers):
-                ax.axvline(
-                    marker[j],
-                    color='k',
-                    linestyle='--',
-                    zorder=420,
-                )
-
-                if i != j:
-                    ax.axhline(
-                        marker[i],
-                        color='k',
-                        linestyle='--',
-                        zorder=420,
-                    )
-
-                    ax.plot(
-                        marker[j], marker[i],
-                        color='k',
-                        marker='s',
-                        zorder=420,
-                    )
-
             ## Ticks
             if i == D - 1:
                 ax.xaxis.set_major_locator(mpl.ticker.MaxNLocator(3, prune='both'))
                 plt.setp(
                     ax.get_xticklabels(),
-                    rotation=45.,
+                    rotation=45.0,
                     horizontalalignment='right',
                     rotation_mode='anchor',
                 )
@@ -379,39 +351,94 @@ def corner(
             ax.label_outer()
 
     figure.align_labels()
+    figure.tight_layout(pad=0.5)
 
     return figure
 
 
-def rank_ecdf(
-    ranks: Array,
+def mark_point(
+    figure: mpl.figure.Figure,
+    point: Array,
+    color: Union[str, tuple] = 'black',
+) -> None:
+    r"""Marks a point on the histograms of a corner plot.
+
+    Arguments:
+        figure: The corner plot figure (see :func:`corner`).
+        point: The point to mark.
+        color: A color for the lines and markers.
+
+    Example:
+        >>> mark_point(figure, [0.5, 0.3, -0.7], color='black')
+
+    .. image:: ../images/corner_marked.png
+        :align: center
+        :width: 600
+    """
+
+    D = len(point)
+    axes = np.asarray(figure.axes).reshape(D, D)
+
+    for i in range(D):
+        for j in range(i + 1):
+            ax = axes[i, j]
+
+            ax.axvline(
+                point[j],
+                color=color,
+                linestyle='--',
+                zorder=420,
+            )
+
+            if i != j:
+                ax.axhline(
+                    point[i],
+                    color=color,
+                    linestyle='--',
+                    zorder=420,
+                )
+
+                ax.plot(
+                    point[j],
+                    point[i],
+                    color=color,
+                    marker='s',
+                    zorder=420,
+                )
+
+
+def coverage_plot(
+    levels: Array,
+    coverages: Array,
     color: Union[str, tuple] = None,
     legend: str = None,
     figure: mpl.figure.Figure = None,
     **kwargs,
 ) -> mpl.figure.Figure:
-    r"""Draws the empirical cumulative distribution function (ECDF) of a rank
-    statistic :math:`r \in [0, 1]`.
+    r"""Plots the expected coverage at various credible levels.
 
     Arguments:
-        ranks: Samples of the rank statistic.
-        color: A color.
+        levels: A vector of increasing credible levels.
+        coverages: A vector of corresponding expected coverages.
         legend: A legend.
-        figure: A ECDF plot over which to draw the new one.
+        figure: Another coverage plot over which to draw the new one.
         kwargs: Keyword arguments passed to :func:`matplotlib.pyplot.subplots`.
 
     Returns:
-        The figure instance for the ECDF plot.
+        The figure instance for the coverage plot.
 
     Example:
-        >>> ranks = np.random.rand(2**12)**2
-        >>> rank_ecdf(ranks)
+        >>> levels = np.linspace(0, 1, 512) ** 2
+        >>> coverages = np.linspace(0, 1, 512)
+        >>> figure = coverage_plot(levels, coverages)
 
-    .. image:: ../static/images/rank_ecdf.png
+    .. image:: ../images/coverage.png
         :align: center
         :width: 400
     """
 
+    levels, coverages = np.asarray(levels), np.asarray(coverages)
+
     # Figure
     if figure is None:
         kwargs.setdefault('figsize', (3.2, 3.2))
@@ -419,25 +446,22 @@ def rank_ecdf(
         figure, ax = plt.subplots(**kwargs)
         new = True
     else:
-        ax = figure.axes.squeeze()
+        ax = figure.axes[0]
         new = False
 
-    # ECDF
-    ranks = np.sort(np.asarray(ranks))
-    ranks = np.hstack([0, ranks, 1])
-    ecdf = np.linspace(0, 1, len(ranks))
-
     # Plot
     if new:
         ax.plot([0, 1], [0, 1], color='k', linestyle='--')
 
-    ax.plot(ranks, ecdf, color=color, label=legend)
+    ax.plot(levels, coverages, color=color, label=legend)
 
     ax.grid()
-    ax.set_xlabel(r'$r$')
-    ax.set_ylabel(r'$\mathrm{ECDF}(r)$')
+    ax.set_xlabel(r'Credible level')
+    ax.set_ylabel(r'Expected coverage')
 
     if legend is not None:
         ax.legend(loc='upper left')
 
+    figure.tight_layout(pad=0.5)
+
     return figure
